#ifndef _PHPFILES_H_
#define _PHPFILES_H_

#include "rulesystem.h"
#include "htmlbox.h"
#include "transformerset.h"
#include "treeparser.h"
#include <fstream>

/// Changes the file extension form php to tpl.
static string phpName2tplName(const string phpName){
  string x = phpName;
  return x.replace(phpName.length()-3,phpName.length(),"tpl");  
};

/// Stores informations about all refactored files. 
class PHPFiles{
 private:

  /// Information about a file
  class PHPFileInfo{
  public:
    /// input file name
    const string phpName;
    /// output php
    StatementList *phpOut;
    /// output tpl
    StatementList *tplOut;
    /// Constructor
    PHPFileInfo(const string iphpName) : phpName(iphpName), 
      phpOut(NULL), tplOut(NULL) {};
  };

  /// Rule system
  RuleSystem factor;

  /// Directory the files are taken from.
  string inDir;
  /// Directory the files are stored to.
  string outDir;

  /// List of file infos.
  typedef list<PHPFileInfo> PHPFileInfoList;
  /// PHPFileInfoList iterator
  typedef PHPFileInfoList::iterator iterator;
  /// PHPFileInfoList iterator
  typedef PHPFileInfoList::const_iterator const_iterator;

  /// Information about refactored php files
  PHPFileInfoList phpFileInfos;

  /// Box for storing html and extracting similar code.
  HTMLBox htmlBox;

  /// If true parser debug messages are printed
  int yydebug;

  /// Generates php file header -- smarty initialization
  void phpHeader(StatementList *phpOut) const;

  /// Generates php file footer -- smarty dispaly
  void phpFooter(StatementList *phpOut, const string tplName) const;
  
  /// Executes ::RuleSystem on all files.
  void executeRuleSystemAtTime(){
    iterator it = phpFileInfos.begin();
    iterator eit = phpFileInfos.end();
    while (it!=eit){
      StatementList *phpIn = it->phpOut;
      it->phpOut = new StatementList();
      cout << "Executing rules at time " << factor.getRuleRunTime() << " on " << it->phpName << endl;
      factor.execute(it->phpName, phpIn, it->phpOut, it->tplOut);
      it++;
    };
  };

  /// Executes ::RuleSystem on all files for all times.
  void executeRuleSystem(){
    do {
      executeRuleSystemAtTime();
    } while (factor.stepTime());
  };
  
  /// Loads and normalizes the files
  void loadIn(){
    iterator it = phpFileInfos.begin();
    iterator eit = phpFileInfos.end();
    while (it!=eit){
      it->tplOut = new StatementList();
      cout << "Reading tree of: " << inDir + it->phpName << endl;
      it->phpOut = phpParser->parse(inDir + it->phpName, yydebug);
      cout << "Normalizing" << endl;
      phpPreNormalizers.apply(it->phpOut);
      it++;
    };
  };

  /// Saves refactored code to files
  void saveOut(){
    iterator it = phpFileInfos.begin();
    iterator eit = phpFileInfos.end();
    while (it!=eit){
      phpHeader(it->phpOut);
      phpFooter(it->phpOut,phpName2tplName(it->phpName));

      phpPostNormalizers.apply(it->phpOut);
      tplPostNormalizers.apply(it->tplOut);

      ofstream phpOutOfs((outDir+it->phpName).c_str());
      phpOutOfs << it->phpOut;
      ofstream tplOutOfs((outDir+phpName2tplName(it->phpName)).c_str());
      tplOutOfs << it->tplOut;
      it++;
    };
  };

  /// Finds similar parts of html
  void processHTML(){
    iterator it = phpFileInfos.begin();
    iterator eit = phpFileInfos.end();
    while (it!=eit){
      it->tplOut->transform(&htmlBox.putter);
      it++;
    };    

    it = phpFileInfos.begin();
    eit = phpFileInfos.end();
    while (it!=eit){
      it->tplOut->transform(&htmlBox.replacer);
      it++;
    };   
  };


 public:

  /// Constructor
  /*!
    Reads configuration file
   */
  PHPFiles(istream &phpConfig, int iyydebug = 0) : yydebug(iyydebug) {
    phpConfig >> inDir;
    phpConfig >> outDir;

    htmlBox.setOutDir(outDir);

    while (!phpConfig.eof()){
      string phpName;
      phpConfig >> phpName;
      if (phpName != "") phpFileInfos.push_back(PHPFileInfo(phpName));
    };
  };

  /// Refactors all files.
  void execute(){
    cout << "Stage 1: reading files" << endl;
    loadIn();
    cout << "Stage 2: executing rule system" << endl;
    executeRuleSystem();
    cout << "Stage 3: processing HTML" << endl;
    processHTML();
    cout << "Stage 4: saving results" << endl;
    saveOut();
    cout << "DONE!!!" << endl;
  };

  /// Destructor
  ~PHPFiles(){
    iterator it = phpFileInfos.begin();
    iterator eit = phpFileInfos.end();
    while (it!=eit){
      delete it->phpOut;
      delete it->tplOut;
      it++;
    };
  };

};

#endif
