#ifndef _RULECONTEXT_H_
#define _RULECONTEXT_H_

#include "stree.h"
#include "rtree.h"
#include "namespace.h"

#define MAINCONTEXTNAME "main"

/// %Context in that rules are executed.
/*!
  Remembers the list of rules and executes tham on php.
  Takes care of assinging names to variables exported to template. 
*/
class RuleContext{
 protected:
  /// List of rules. 
  RuleList &rules;

 private:

  /// List of rules belonging to the context.
  /*!
    Every inserted rule is put to RuleContext::rules and an
    iterator is stored in myRules. When the context is destroyed the 
    stored rules are removed from RuleContext::rules.
   */
  list<RuleList::iterator> myRules;

  /// Namespace

  /// Tries to execute rules on php.
  /*!
    \param phpIn input php
    \param phpOut output php
    \param tplOut output smarty template
    
    Douring the execution the begining of \a phpIn is destroyed when the rules
    match.
   */
  bool executeOnPHP(List* phpIn, List* phpOut, List* tplOut);

  /// Executes executeOnPHP unless the phpIn is empty.
  void executeOnPHPList(List* phpIn, List *phpOut, List *tplOut);

  /// Sets appropriate strings in appearing ::SmartyAssing classes.
  void processSmartyAssignInPhpList(List* phpList);

 protected:
  /// A name space in which rules are executed
  NameSpace &nameSpace;

 public:

  /// Constructes a context with given rules
  RuleContext(RuleList &irules, NameSpace &inameSpace);
  
  /// Destroys the context
  virtual ~RuleContext();

  /// Inserts a Rule with given priority in to the context.
  void insert(int priority, Rule *rule);

  /// Inserts a Rule with given priority in to the context.
  void insert(PriorityRulePair *rulePair);

  /// Executes all rules of the context on php.
  /*!
    \param phpIn input php
    \param phpOut output php
    \param tplOut output smarty template
    
    During the execution \a phpIn is destroyed.
   */
  void execute(Node* phpIn,List *phpOut, List *tplOut);

  /// Assigns appropriate names to variables appearing in given lists. 
  void assignVariableNames(List *phpList, List *tplList);
    
  /// Generates appropriate names to variables and stores them in assignments.
  virtual void generateVariableNames(const Node::VariableList &vlist, Node::Assignment &phpAssign, Node::Assignment &tplAssign) = 0;

  /// Generates one variable name.
  virtual const string generateVariableName() = 0;

  /// Processes one SmartyAssign.
  virtual void processSmartyAssign(SmartyAssign *smartyAssign) = 0;

  /// Run at the begining of RuleContext::execute.
  virtual void enterContext(List *phpOut, List *tplOut);
  /// Run at the end of RuleContext::execute.
  /*!
    When living the context all registerd smarty variables are exported.
   */
  virtual void leaveContext(List *phpOut, List *tplOut);

  /// Registers smarty varaible in the context.
  virtual void registerSmartyVariable(const string &variable) = 0;

  /// Returns a sufix for of a varaible name.
  /*!
    In case of control variables e.g. in for statement, a leading dollar may be skiped, when
    exporting control variable to template.
   */
  virtual const string getVariableNameSufix(const bool stripControlVariableDollar) = 0;
  /// Returns a sufix for of a varaible name.
  virtual const string getVariableNamePrefix() = 0;

  /// Processes template after refactoring
  virtual void processTemplate(Node* tpl) const = 0;

  /// Returns nameContext
  NameSpace& getNameSpace() const { return nameSpace; };

  friend class MultiContext;
  friend class SingleContext;
};

#endif



