#ifndef _STREE_NODE_H_
#define _STREE_NODE_H_

#include "location.h"

#include <map>
#include <string>
#include <set>
#include <iostream>

#ifndef DOXYSKIP
using namespace std;
#endif

/// PHP structure tree node types
enum SType {
  SVariable, //!< Node of this type can be matched to any other subtree.
  SConstant, //!< Node containing a constant string.
  SList,     //!< Node having many children
  SPin,      //!< A smarter NULL.
  SContext   //!< Context sensitive node -- after a subtree was matched checks some conditions ctree.h. 
};

class NameSpace;
class List;

/// A basis class for PHP structure tree.
class Node
{
 protected:
  /// The location of the node -- not used
  Location location;

public:
  /// An assignment -- map from variables to trees.
  class Assignment : public map<int,const Node*>{
  public:
    ~Assignment(){
      iterator it = begin();
      iterator eit = end();
      while (it!=eit){
	delete it->second;
	it++;
      };
    };
  };

  /// Defines a method for transforming nodes used by Node::transform.
  class Transformer {
  public:
    virtual Node* transform(Node* node) const = 0;
  };
  
  /// Defines a method for testing nodes used by Node::forall, Node::exists.
  class Tester {
  public:
    virtual bool test(const Node* node, const NameSpace &nameSpace) const = 0;
    virtual Tester* copy() const = 0;
    virtual Tester* substitute(const Assignment &assign) = 0;
    virtual ~Tester() {};
  };

  /// A list of variables.
  typedef set<int> VariableList;
  
  /// Writes assignment to stream
  friend ostream& operator<<(ostream &os, const Assignment &data);

  /// Constructor
  Node(const string &icomment, const int ilineno = 0) : location(icomment,ilineno) {}
  
  /// Constructor 
  Node() : location("",0) {};

  /// Destrys the node and frees memory.
  virtual ~Node() {};

  /// Returns type of the node.
  virtual SType getType() const = 0;    

  /// Returs string representing node type or name for ::List
  virtual string getName() const = 0;
  
  /// Writes node to the stream.
  virtual ostream& print(ostream &os) const = 0;

  /// Writes node to the stream as php.
  virtual ostream& php(ostream &os) const = 0;

  /// Matches node \a p to the node. If node matches sets the assignment. 
  virtual bool match(const Node *p, Assignment &assign, const NameSpace &nameSpace) const = 0;

  /// Matches the list to the begining of \a p
  virtual bool matchToBegining(const List *p, Assignment &assign, const NameSpace &nameSpace) const = 0;

  /// Comares node \a p to the node. 
  virtual bool compare(const Node *p) const = 0;

  /// Copies the node. 
  virtual Node* copy() const = 0;
  
  /// Returns list of variables used in the node. 
  virtual void getVariables(VariableList &vlist) const = 0;    
  
  /// Writes the node to the stream by using Node::php
  friend ostream& operator<<(ostream &os, const Node *data);
  
  /// Returns the string representing the node
  string toString() const; 
  
  /// Correct only for lists. 
  virtual Node* push_back(Node *statement);
  
  /// Applies Transformer to every node in the tree.
  virtual Node* transform(const Transformer* transformer);
  
  /// Returns true is for all nodes in the 
  virtual bool forall(const Tester* tester, const NameSpace &nameSpace) const;
  
  /// Applies Transformer to every node in the tree.
  virtual bool exists(const Tester* tester, const NameSpace &nameSpace) const;

  /// Substitutes variables in the node accordingly to the given assignment.
  friend void substituteNode(Node* &node, const Assignment &assign);

  /// Computes the value of the expression (only basic atihmetic and string operations)
  virtual string evaluate(const NameSpace &nameSpace) const = 0;

  /// Returns teh length of the list -- valid only for ::List and ::ContextMatch
  virtual int getLength() const;

 protected:
  /// Substitutes variables in the node accordingly to the given assignment.
  virtual Node* substitute(const Assignment &assign) = 0; 
};

#endif

